/*

   SHARED.h - include file for VxD BILLGPS

   Free software Copyright (c) 1996 Lew Perin

   Contains the definition for what's shared between billgpc.exe
   and billgps.vxd

*/

#define ONLY_BOOTP 1            /* Accepting only bootp replies? */

enum { MacAddressSize = 6 };

/*

   W32 IOCTL command definitions: First we declare the raw function codes
   and then, with the aid of the CTL_CODE macro, the finished IOCTL codes.

*/

enum {
  FnCode_BillgpAwaitBootp = 0x800,
  FnCode_BillgpAwaitAny,
  FnCode_BillgpGetVersion,
  FnCode_BillgpGetMacAddress,
  FnCode_BillgpDumpDriver,
  FnCode_BillgpGetHWType,
  FnCode_BillgpBind
};

enum {
  DIOC_BillgpAwaitBootp =
    CTL_CODE(FILE_DEVICE_UNKNOWN, FnCode_BillgpAwaitBootp,
             METHOD_BUFFERED, FILE_ANY_ACCESS),
  DIOC_BillgpAwaitAny =
    CTL_CODE(FILE_DEVICE_UNKNOWN, FnCode_BillgpAwaitAny,
             METHOD_BUFFERED, FILE_ANY_ACCESS),
  DIOC_BillgpGetVersion =
    CTL_CODE(FILE_DEVICE_UNKNOWN, FnCode_BillgpGetVersion,
             METHOD_BUFFERED, FILE_ANY_ACCESS),
  DIOC_BillgpGetMacAddress =
    CTL_CODE(FILE_DEVICE_UNKNOWN, FnCode_BillgpGetMacAddress,
             METHOD_BUFFERED, FILE_ANY_ACCESS),
  DIOC_BillgpDumpDriver =
    CTL_CODE(FILE_DEVICE_UNKNOWN, FnCode_BillgpDumpDriver,
             METHOD_BUFFERED, FILE_ANY_ACCESS),
  DIOC_BillgpGetHWType =
    CTL_CODE(FILE_DEVICE_UNKNOWN, FnCode_BillgpGetHWType,
             METHOD_BUFFERED, FILE_ANY_ACCESS),
  DIOC_BillgpBind =
    CTL_CODE(FILE_DEVICE_UNKNOWN, FnCode_BillgpBind,
             METHOD_BUFFERED, FILE_ANY_ACCESS)
};

/*
   Ethernet headers from W. R. Stevens, TCP/IP Illustrated, vol. 1,
   Fig. 2.1
*/

typedef struct {
  UCHAR destination[MacAddressSize];
  UCHAR source[MacAddressSize];
  USHORT packetType;
} RFC894Header;

typedef struct {
  UCHAR dsap;
  UCHAR ssap;
  UCHAR cntl;
  UCHAR orgCode[3];
  USHORT packetType;
} LLCSNAP;

typedef struct {
  UCHAR destination[MacAddressSize];
  UCHAR source[MacAddressSize];
  USHORT length;
  LLCSNAP llcSnap;
} RFC1042Header;

/*
   Note that packet types have been expressed in Intel byte order.
*/

enum {
  PacketTypeIP = 8
};

typedef struct {
  UCHAR versionAndHeaderLength;
  UCHAR tos;
  USHORT totalLength;
  USHORT identification;
  USHORT flagsAndFragmentOffset;
  UCHAR ttl;
  UCHAR protocol;
  USHORT checksum;
  ULONG sourceAddr;
  ULONG destinationAddr;
} IPHeader;

typedef struct {
  USHORT sourcePort;
  USHORT destinationPort;
  USHORT udpLength;
  USHORT udpChecksum;
} UDPHeader;

enum {
  RFC951HaddrLen = 16, // Hardware address length
  RFC951BOOTREQUEST = 1, RFC951BOOTREPLY = 2, // "op" byte in bootp buffer
  RFC951VendSize = 64,                        // size of vendor-specific area
  RFC1084VendorMagicCookie = 0x63825363 // Can we do the RFC 1084 extensions?
};

enum BootpHWType {
  BootpHWTypeEthernet = 1, BootpHWTypeIEEE = 6
};

enum BootpFlags {
  BootpFlagBroadcast = 0x8000
};

typedef struct {
  UCHAR op;                     // packet op code / message type
  UCHAR htype;                  // h/w address type, 1 = 10 mb ethernet
  UCHAR hlen;                   // h/w address len, eg '6' for 10mb eth
  UCHAR hops;                   // client sets to zero, optionally used by
                                // gateways in cross-gateway booting.
  ULONG xid;                    // transaction ID, a random number
  USHORT secs;                  // filled in by client, seconds elapsed since
                                // client started trying to boot
  USHORT flags;                 // RFC 1542 broadcast
  ULONG ciaddr;                 // client IP address filled in by client
                                // if known
  ULONG yiaddr;                 // 'your' (client) IP address
                                // filled by server if client doesn't know
  ULONG siaddr;                 // server IP address returned in bootreply
  ULONG giaddr;                 // gateway IP address,
                                // used in optional cross-gateway booting
  UCHAR chaddr[RFC951HaddrLen]; // client hardware address,
                                // filled by client
  UCHAR sname[64];              // optional server host name,
                                // null terminated
  UCHAR file[128];              // boot file name, null terminated string
                                // 'generic' name or null in bootrequest,
                                // fully qualified directory-path
                                // name in bootreply
  UCHAR vend[RFC951VendSize];   // optional vendor-specific area
} BootpBuf;

/*
   Data structure that contains everything that we
   pass to the app
*/

typedef struct {
#if ONLY_BOOTP
#else
  UINT headerSize;
  UCHAR header[sizeof(RFC1042Header)];
#endif
#ifdef DRIVER_DUMP
  char adapterName[64];
  ULONG contextAtOpen;
  ULONG contextAtNIR;
#endif
#if 0                           // Not needed with 1.24ff
  UINT dataSize;
#endif
  UCHAR data[1500];
} BillgpBuf;

/*

   Externs

*/

extern BillgpBuf billgpBuf;


